from .generators import SchemaGenerator
from .endpoints import Schema, SwaggerUI, RedocUI
from .extensions import Starchart

__all__ = ["SchemaGenerator", "SwaggerUI", "RedocUI", "Schema", "Starchart"]
__version__ = "0.3.0"



"""OpenAPI3 example
"""
import asyncio

from starlette.applications import Starlette
from starlette.requests import Request
from starlette.responses import JSONResponse
from starlette.endpoints import HTTPEndpoint
from uvicorn import run
from openapi_spec_validator import validate_v3_spec
from starlette.testclient import TestClient

from starchart import Starchart


app = Starlette(debug=True)

starchart = Starchart(
    title="Dog store",
    description="Dog store api document",
    version="0.1",
    openapi_version="3.0.0",
)
starchart.register(app)

# define data
DOGS = {1: {"id": 1, "name": "Ping", "age": 2}, 2: {"id": 2, "name": "Pong", "age": 1}}
# add schema
starchart.schema_generator.add_schema(
    "Dog",
    {
        "properties": {
            "id": {"description": "global unique", "type": "integer"},
            "name": {"type": "string"},
            "age": {"type": "integer"},
        },
        "type": "object",
    },
)


# define routes and schema(in doc string)
@app.route("/dog/")
class Dog(HTTPEndpoint):
    def get(self, req: Request):
        """
        summary: Get single dog
        tags:
        - dog
        parameters:
        - name: id
          in: query
          required: True
          schema:
            type: integer
        responses:
          "200":
            description: OK
            content:
              application/json:
                schema:
                  $ref: '#/components/schemas/Dog'
          "404":
            description: Not found
       """
        return JSONResponse(DOGS[1])

    def delete(self, req: Request):
        """
        summary: Delete single dog
        tags:
        - dog
        parameters:
        - name: id
          in: query
          required: True
          schema:
            type: integer
        responses:
          "204":
            description: OK
            content:
              application/json:
                schema:
                  $ref: '#/components/schemas/Dog'
          "404":
            description: Not found
        """
        dog = DOGS.pop(1)
        return JSONResponse(dog)


# define doc by yaml or json file
@app.route("/dogs/", methods=["GET"])
@starchart.schema_generator.schema_from("./examples/docs/dogs_get.yml")
def list_dogs(req: Request):
    return JSONResponse(list(DOGS.values()))


@app.route("/dogs/", methods=["POST"])
@starchart.schema_generator.schema_from("./examples/docs/dogs_post.json")
async def list_dogs(req: Request):
    dog = await req.json()
    DOGS[dog["id"]] = dog
    return JSONResponse(dog)


def test_app():
    # Trigger app start event
    handler = app.lifespan_middleware({"type": "lifespan"})
    asyncio.get_event_loop().run_until_complete(handler.startup())

    client = TestClient(app)
    validate_v3_spec(app.schema)
    assert client.get("/docs/schema/").json() == app.schema
    assert client.get("/docs/swagger/").status_code == 200
    assert client.get("/docs/redoc/").status_code == 200


if __name__ == "__main__":
    run(app)



"""OpenAPI2(Swagger) example
"""
from functools import partial

from starlette.applications import Starlette
from starlette.requests import Request
from starlette.responses import JSONResponse
from starlette.endpoints import HTTPEndpoint
from uvicorn import run
from openapi_spec_validator import validate_v2_spec
from starlette.testclient import TestClient

from starchart.generators import SchemaGenerator
from starchart.endpoints import SwaggerUI, RedocUI, Schema


app = Starlette(debug=True)

app.schema_generator = SchemaGenerator(
    title="Cat store",
    description="Cat store api document",
    version="0.1",
    openapi_version="2.0",
)
# define data
CATS = {
    1: {"id": 1, "name": "DangDang", "age": 2},
    2: {"id": 2, "name": "DingDing", "age": 1},
}
# add schema definition
app.schema_generator.add_schema(
    "Cat",
    {
        "properties": {
            "id": {"description": "global unique", "type": "integer"},
            "name": {"type": "string"},
            "age": {"type": "integer"},
        },
        "type": "object",
    },
)


# define routes and schema(in doc string)
@app.route("/cat/")
class Cat(HTTPEndpoint):
    def get(self, req: Request):
        """
        summary: Get single cat
        tags:
        - cat
        parameters:
        - name: id
          type: integer
          in: query
          required: True
        responses:
          "200":
            description: OK
            schema:
              $ref: '#/definitions/Cat'
          "404":
            description: Not found
       """
        return JSONResponse(CATS[1])

    def delete(self, req: Request):
        """
        summary: Delete single cat
        tags:
        - cat
        parameters:
        - name: id
          type: integer
          in: query
          required: True
        responses:
          "204":
            description: OK
            schema:
              $ref: '#/definitions/Cat'
          "404":
            description: Not found
        """
        cat = CATS.pop(1)
        return JSONResponse(cat)


# define doc by yaml or json file
@app.route("/cats/", methods=["GET"])
@app.schema_generator.schema_from("./examples/docs/cats_get.yml")
def list_cats(req: Request):
    return JSONResponse(list(CATS.values()))


@app.route("/cats/", methods=["POST"])
@app.schema_generator.schema_from("./examples/docs/cats_post.json")
async def list_cats(req: Request):
    cat = await req.json()
    CATS[cat["id"]] = cat
    return JSONResponse(cat)


# add document's endpoints
schema_path = "/docs/schema/"
app.add_route(
    "/docs/swagger/",
    SwaggerUI,
    methods=["GET"],
    name="SwaggerUI",
    include_in_schema=False,
)
app.add_route(
    "/docs/redoc/", RedocUI, methods=["GET"], name="SwaggerUI", include_in_schema=False
)
app.add_route(
    schema_path, Schema, methods=["GET"], name="SwaggerSchema", include_in_schema=False
)
# config endpoints
SwaggerUI.set_schema_url(schema_path)
RedocUI.set_schema_url(schema_path)
Schema.set_schema_loader(partial(app.schema_generator.get_schema, app.routes))


def test_app():
    client = TestClient(app)
    validate_v2_spec(app.schema)
    assert client.get(schema_path).json() == app.schema
    assert client.get("/docs/swagger/").status_code == 200
    assert client.get("/docs/redoc/").status_code == 200


if __name__ == "__main__":
    run(app)



from importlib import reload, import_module
from pkgutil import iter_modules

import starchart
import examples


def test_examples():
    for _, name, _ in iter_modules(examples.__path__):
        example = import_module(f"examples.{name}")
        example.test_app()
        reload(starchart)



import typing
from collections import defaultdict
import json

from starlette.schemas import BaseSchemaGenerator
from starlette.routing import BaseRoute
import yaml


class SchemaGenerator(BaseSchemaGenerator):
    SCHEMA_PATH_ATTR_NAME = "__schema_path__"

    def __init__(
        self,
        title: str = "Starchart",
        version: str = "0.1",
        description: str = "Api document",
        openapi_version: str = "2.0",
    ):
        self.schema: typing.Dict = {
            "info": {"title": title, "description": description, "version": version},
            "paths": defaultdict(dict),
        }
        if openapi_version == "2.0":
            self.schema["swagger"] = openapi_version
            self.schema["definitions"] = {}
        else:
            self.schema["openapi"] = openapi_version
            self.schema["components"] = {"schemas": {}}
        self.loaded = False

    @staticmethod
    def load_document(file_path: str) -> typing.Dict:
        with open(file_path) as f:
            if file_path.endswith("json"):
                return json.load(f)
            else:
                return yaml.safe_load(f)

    def add_schema(self, name, definition):
        if "swagger" in self.schema:
            self.schema["definitions"][name] = definition
        else:
            self.schema["components"]["schemas"][name] = definition

    def add_path(self, path: str, method: str, document: typing.Dict):
        self.schema["paths"][path][method] = document

    def load_schema(self, routes: typing.List[BaseRoute]):
        for endpoint in self.get_endpoints(routes):
            doc_path: typing.Optional[str] = getattr(
                endpoint.func, self.SCHEMA_PATH_ATTR_NAME, None
            )

            if doc_path:
                doc = self.load_document(doc_path)
            else:
                doc = self.parse_docstring(endpoint.func)

            if doc:
                schemas_name = "definitions" if "swagger" in self.schema else "schemas"
                try:
                    schemas = doc.pop(schemas_name)
                except KeyError:
                    schemas = {}
                for name, schema in schemas.items():
                    self.add_schema(name, schema)

                self.add_path(endpoint.path, endpoint.http_method, doc)

        self.loaded = True

    def get_schema(self, routes: typing.List[BaseRoute]) -> typing.Dict:
        if not self.loaded:
            self.load_schema(routes)
        return self.schema

    def schema_from(self, file_path: str) -> typing.Callable:
        def decorator(func: typing.Callable) -> typing.Callable:
            setattr(func, self.SCHEMA_PATH_ATTR_NAME, file_path)
            return func

        return decorator



import typing
from functools import partial

from starlette.applications import Starlette

from .endpoints import SwaggerUI, RedocUI, Schema
from .generators import SchemaGenerator


class Starchart:
    """A extension to handle starchart's config"""

    def __init__(
        self,
        title="Starchart",
        description="Api document",
        version="0.1",
        openapi_version="3.0.0",
        schema_path="/docs/schema/",
        doc_path="/docs/",
    ):
        self.schema_generator = SchemaGenerator(
            title=title,
            description=description,
            version=version,
            openapi_version=openapi_version,
        )
        SwaggerUI.set_schema_url(schema_path)
        RedocUI.set_schema_url(schema_path)
        self.doc_path = doc_path
        self.schema_path = schema_path
        self.app: typing.Optional[Starlette] = None

    def register(self, app: Starlette):
        self.app = app
        app.add_event_handler("startup", self.startup)

    def startup(self):
        self.schema_generator.load_schema(self.app.routes)
        Schema.set_schema_loader(
            partial(self.schema_generator.get_schema, self.app.routes)
        )

        self.app.add_route(
            self.schema_path,
            Schema,
            methods=["GET"],
            name="SwaggerSchema",
            include_in_schema=False,
        )
        self.app.add_route(
            f"{self.doc_path}swagger/",
            SwaggerUI,
            methods=["GET"],
            name="SwaggerUI",
            include_in_schema=False,
        )
        self.app.add_route(
            f"{self.doc_path}redoc/",
            RedocUI,
            methods=["GET"],
            name="SwaggerUI",
            include_in_schema=False,
        )
        self.app.schema_generator = self.schema_generator



#!/usr/bin/env python3
from setuptools import setup, find_packages
import re

with open("starchart/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r"__version__ = \"(.*?)\"", f.read()).group(1)

requires = ["starlette>=0.9.9", "PyYAML>=3.13", "Jinja2>=2.10"]
tests_requires = [
    "pytest>=3.3.1",
    "pytest-cov>=2.5.1",
    "openapi_spec_validator>=0.2.4",
    "jsonschema<3",
    "uvicorn>=0.3.24",
]
setup_requires = ["pytest-runner>=3.0", "black"]

setup(
    name="starchart",
    version=version,
    url="https://github.com/strongbugman/starchart",
    description="Starlette's api document support",
    long_description=open("README.md").read(),
    author="Bruce Wu",
    author_email="strongbugman@gmail.com",
    license="BSD",
    classifiers=[
        "Environment :: Console",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    packages=find_packages(exclude=("tests", "tests.*", "examples")),
    install_requires=requires,
    setup_requires=setup_requires,
    tests_require=tests_requires,
    include_package_data=True,
)



import typing

from starlette.endpoints import HTTPEndpoint
from starlette.requests import Request
from starlette.responses import TemplateResponse, JSONResponse
from jinja2 import Template

import starchart


class UI(HTTPEndpoint):
    TEMPLATE = Template(open(f"{starchart.__path__[0]}/static/index.html").read())
    CONTEXT = {"title": "Starchart", "schema_url": "./schema/", "request": None}

    def get(self, res: Request) -> TemplateResponse:
        self.CONTEXT["request"] = res
        return TemplateResponse(self.TEMPLATE, self.CONTEXT)

    @classmethod
    def set_template(cls, path: str) -> None:
        cls.TEMPLATE = Template(open(path).read())

    @classmethod
    def set_schema_url(cls, schema_url: str) -> None:
        cls.CONTEXT["schema_url"] = schema_url

    @classmethod
    def set_title(cls, tilte: str) -> None:
        cls.CONTEXT["title"] = tilte


class SwaggerUI(UI):
    TEMPLATE = Template(
        open(f"{starchart.__path__[0]}/static/swagger_index.html").read()
    )


class RedocUI(UI):
    TEMPLATE = Template(open(f"{starchart.__path__[0]}/static/redoc_index.html").read())


class Schema(HTTPEndpoint):
    SCHEMA_LOADER: typing.Callable[[], typing.Dict] = lambda: dict

    def get(self, res: Request) -> JSONResponse:
        return JSONResponse(self.SCHEMA_LOADER())

    @classmethod
    def set_schema_loader(cls, loader: typing.Callable[[], typing.Dict]) -> None:
        cls.SCHEMA_LOADER = loader
